# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from kater import Kater, AsyncKater
from tests.utils import assert_matches_type
from kater.types.v1 import (
    GitHubConnectResponse,
    GitHubGetStatusResponse,
    GitHubGetInstallationLinkResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestGitHub:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_callback(self, client: Kater) -> None:
        github = client.v1.github.callback(
            state="state",
        )
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_callback_with_all_params(self, client: Kater) -> None:
        github = client.v1.github.callback(
            state="state",
            code="code",
            error="error",
            error_description="error_description",
            installation_id=0,
        )
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_callback(self, client: Kater) -> None:
        response = client.v1.github.with_raw_response.callback(
            state="state",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = response.parse()
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_callback(self, client: Kater) -> None:
        with client.v1.github.with_streaming_response.callback(
            state="state",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = response.parse()
            assert_matches_type(object, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_connect(self, client: Kater) -> None:
        github = client.v1.github.connect()
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_connect_with_all_params(self, client: Kater) -> None:
        github = client.v1.github.connect(
            return_to="return_to",
        )
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_connect(self, client: Kater) -> None:
        response = client.v1.github.with_raw_response.connect()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = response.parse()
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_connect(self, client: Kater) -> None:
        with client.v1.github.with_streaming_response.connect() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = response.parse()
            assert_matches_type(GitHubConnectResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_installation_link(self, client: Kater) -> None:
        github = client.v1.github.get_installation_link()
        assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_installation_link(self, client: Kater) -> None:
        response = client.v1.github.with_raw_response.get_installation_link()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = response.parse()
        assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_installation_link(self, client: Kater) -> None:
        with client.v1.github.with_streaming_response.get_installation_link() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = response.parse()
            assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_get_status(self, client: Kater) -> None:
        github = client.v1.github.get_status()
        assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_get_status(self, client: Kater) -> None:
        response = client.v1.github.with_raw_response.get_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = response.parse()
        assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_get_status(self, client: Kater) -> None:
        with client.v1.github.with_streaming_response.get_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = response.parse()
            assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncGitHub:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_callback(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.callback(
            state="state",
        )
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_callback_with_all_params(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.callback(
            state="state",
            code="code",
            error="error",
            error_description="error_description",
            installation_id=0,
        )
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_callback(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.github.with_raw_response.callback(
            state="state",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = await response.parse()
        assert_matches_type(object, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_callback(self, async_client: AsyncKater) -> None:
        async with async_client.v1.github.with_streaming_response.callback(
            state="state",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = await response.parse()
            assert_matches_type(object, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_connect(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.connect()
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_connect_with_all_params(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.connect(
            return_to="return_to",
        )
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_connect(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.github.with_raw_response.connect()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = await response.parse()
        assert_matches_type(GitHubConnectResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_connect(self, async_client: AsyncKater) -> None:
        async with async_client.v1.github.with_streaming_response.connect() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = await response.parse()
            assert_matches_type(GitHubConnectResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_installation_link(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.get_installation_link()
        assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_installation_link(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.github.with_raw_response.get_installation_link()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = await response.parse()
        assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_installation_link(self, async_client: AsyncKater) -> None:
        async with async_client.v1.github.with_streaming_response.get_installation_link() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = await response.parse()
            assert_matches_type(GitHubGetInstallationLinkResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_get_status(self, async_client: AsyncKater) -> None:
        github = await async_client.v1.github.get_status()
        assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_get_status(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.github.with_raw_response.get_status()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        github = await response.parse()
        assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_get_status(self, async_client: AsyncKater) -> None:
        async with async_client.v1.github.with_streaming_response.get_status() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            github = await response.parse()
            assert_matches_type(GitHubGetStatusResponse, github, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.github import repo_select_params
from ....types.v1.github.repo_list_response import RepoListResponse
from ....types.v1.github.repo_select_response import RepoSelectResponse

__all__ = ["ReposResource", "AsyncReposResource"]


class ReposResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReposResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ReposResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReposResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return ReposResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RepoListResponse:
        """List repositories accessible to the GitHub App installation."""
        return self._get(
            "/api/v1/github/repos",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RepoListResponse,
        )

    def select(
        self,
        repo_id: int,
        *,
        use_existing_structure: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RepoSelectResponse:
        """
        Select a repository for the Kater workspace.

        Args:
          repo_id: GitHub repository ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            f"/api/v1/github/repos/{repo_id}/select",
            body=maybe_transform(
                {"use_existing_structure": use_existing_structure}, repo_select_params.RepoSelectParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RepoSelectResponse,
        )


class AsyncReposResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReposResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncReposResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReposResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncReposResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RepoListResponse:
        """List repositories accessible to the GitHub App installation."""
        return await self._get(
            "/api/v1/github/repos",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RepoListResponse,
        )

    async def select(
        self,
        repo_id: int,
        *,
        use_existing_structure: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RepoSelectResponse:
        """
        Select a repository for the Kater workspace.

        Args:
          repo_id: GitHub repository ID

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            f"/api/v1/github/repos/{repo_id}/select",
            body=await async_maybe_transform(
                {"use_existing_structure": use_existing_structure}, repo_select_params.RepoSelectParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RepoSelectResponse,
        )


class ReposResourceWithRawResponse:
    def __init__(self, repos: ReposResource) -> None:
        self._repos = repos

        self.list = to_raw_response_wrapper(
            repos.list,
        )
        self.select = to_raw_response_wrapper(
            repos.select,
        )


class AsyncReposResourceWithRawResponse:
    def __init__(self, repos: AsyncReposResource) -> None:
        self._repos = repos

        self.list = async_to_raw_response_wrapper(
            repos.list,
        )
        self.select = async_to_raw_response_wrapper(
            repos.select,
        )


class ReposResourceWithStreamingResponse:
    def __init__(self, repos: ReposResource) -> None:
        self._repos = repos

        self.list = to_streamed_response_wrapper(
            repos.list,
        )
        self.select = to_streamed_response_wrapper(
            repos.select,
        )


class AsyncReposResourceWithStreamingResponse:
    def __init__(self, repos: AsyncReposResource) -> None:
        self._repos = repos

        self.list = async_to_streamed_response_wrapper(
            repos.list,
        )
        self.select = async_to_streamed_response_wrapper(
            repos.select,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.connections import database_update_params, database_update_schema_params
from ....types.v1.connections.database_update_response import DatabaseUpdateResponse
from ....types.v1.connections.database_update_schema_response import DatabaseUpdateSchemaResponse

__all__ = ["DatabasesResource", "AsyncDatabasesResource"]


class DatabasesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DatabasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return DatabasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DatabasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return DatabasesResourceWithStreamingResponse(self)

    def update(
        self,
        database_id: str,
        *,
        connection_id: str,
        auto_merge: bool | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseUpdateResponse:
        """Update a database's metadata.

        Updates label and/or description fields directly.

        If a name change is requested,
        a GitHub PR is created to update view YAML files. The name change is applied to
        the DB when the PR merges (or immediately if auto_merge is true or no views are
        affected).

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: DatabaseNotFoundError: If database doesn't exist (404)

        Args:
          auto_merge: If true and a name change requires a PR, auto-merge it

          description: Database description

          label: Human-readable display label

          name: Database name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return self._patch(
            f"/api/v1/connections/{connection_id}/databases/{database_id}",
            body=maybe_transform(
                {
                    "auto_merge": auto_merge,
                    "description": description,
                    "label": label,
                    "name": name,
                },
                database_update_params.DatabaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatabaseUpdateResponse,
        )

    def delete_schema(
        self,
        schema_id: str,
        *,
        connection_id: str,
        database_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a schema from a connection.

        Soft-deletes the schema record, setting deleted_at and deleted_by fields. The
        connection_id and database_id path parameters provide context for the resource
        hierarchy, though the schema_id alone uniquely identifies the record.

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: SchemaNotFoundError: If schema doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/v1/connections/{connection_id}/databases/{database_id}/schemas/{schema_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def update_schema(
        self,
        schema_id: str,
        *,
        connection_id: str,
        database_id: str,
        auto_merge: bool | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseUpdateSchemaResponse:
        """Update a schema's metadata.

        Updates label and/or description fields directly.

        If a name change is requested,
        a GitHub PR is created to update view YAML files. The name change is applied to
        the DB when the PR merges (or immediately if auto_merge is true or no views are
        affected).

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: SchemaNotFoundError: If schema doesn't exist (404)

        Args:
          auto_merge: If true and a name change requires a PR, auto-merge it

          description: Schema description

          label: Human-readable display label

          name: Schema name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return self._patch(
            f"/api/v1/connections/{connection_id}/databases/{database_id}/schemas/{schema_id}",
            body=maybe_transform(
                {
                    "auto_merge": auto_merge,
                    "description": description,
                    "label": label,
                    "name": name,
                },
                database_update_schema_params.DatabaseUpdateSchemaParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatabaseUpdateSchemaResponse,
        )


class AsyncDatabasesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDatabasesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncDatabasesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDatabasesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncDatabasesResourceWithStreamingResponse(self)

    async def update(
        self,
        database_id: str,
        *,
        connection_id: str,
        auto_merge: bool | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseUpdateResponse:
        """Update a database's metadata.

        Updates label and/or description fields directly.

        If a name change is requested,
        a GitHub PR is created to update view YAML files. The name change is applied to
        the DB when the PR merges (or immediately if auto_merge is true or no views are
        affected).

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: DatabaseNotFoundError: If database doesn't exist (404)

        Args:
          auto_merge: If true and a name change requires a PR, auto-merge it

          description: Database description

          label: Human-readable display label

          name: Database name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        return await self._patch(
            f"/api/v1/connections/{connection_id}/databases/{database_id}",
            body=await async_maybe_transform(
                {
                    "auto_merge": auto_merge,
                    "description": description,
                    "label": label,
                    "name": name,
                },
                database_update_params.DatabaseUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatabaseUpdateResponse,
        )

    async def delete_schema(
        self,
        schema_id: str,
        *,
        connection_id: str,
        database_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a schema from a connection.

        Soft-deletes the schema record, setting deleted_at and deleted_by fields. The
        connection_id and database_id path parameters provide context for the resource
        hierarchy, though the schema_id alone uniquely identifies the record.

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: SchemaNotFoundError: If schema doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/v1/connections/{connection_id}/databases/{database_id}/schemas/{schema_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def update_schema(
        self,
        schema_id: str,
        *,
        connection_id: str,
        database_id: str,
        auto_merge: bool | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DatabaseUpdateSchemaResponse:
        """Update a schema's metadata.

        Updates label and/or description fields directly.

        If a name change is requested,
        a GitHub PR is created to update view YAML files. The name change is applied to
        the DB when the PR merges (or immediately if auto_merge is true or no views are
        affected).

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: SchemaNotFoundError: If schema doesn't exist (404)

        Args:
          auto_merge: If true and a name change requires a PR, auto-merge it

          description: Schema description

          label: Human-readable display label

          name: Schema name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        if not database_id:
            raise ValueError(f"Expected a non-empty value for `database_id` but received {database_id!r}")
        if not schema_id:
            raise ValueError(f"Expected a non-empty value for `schema_id` but received {schema_id!r}")
        return await self._patch(
            f"/api/v1/connections/{connection_id}/databases/{database_id}/schemas/{schema_id}",
            body=await async_maybe_transform(
                {
                    "auto_merge": auto_merge,
                    "description": description,
                    "label": label,
                    "name": name,
                },
                database_update_schema_params.DatabaseUpdateSchemaParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DatabaseUpdateSchemaResponse,
        )


class DatabasesResourceWithRawResponse:
    def __init__(self, databases: DatabasesResource) -> None:
        self._databases = databases

        self.update = to_raw_response_wrapper(
            databases.update,
        )
        self.delete_schema = to_raw_response_wrapper(
            databases.delete_schema,
        )
        self.update_schema = to_raw_response_wrapper(
            databases.update_schema,
        )


class AsyncDatabasesResourceWithRawResponse:
    def __init__(self, databases: AsyncDatabasesResource) -> None:
        self._databases = databases

        self.update = async_to_raw_response_wrapper(
            databases.update,
        )
        self.delete_schema = async_to_raw_response_wrapper(
            databases.delete_schema,
        )
        self.update_schema = async_to_raw_response_wrapper(
            databases.update_schema,
        )


class DatabasesResourceWithStreamingResponse:
    def __init__(self, databases: DatabasesResource) -> None:
        self._databases = databases

        self.update = to_streamed_response_wrapper(
            databases.update,
        )
        self.delete_schema = to_streamed_response_wrapper(
            databases.delete_schema,
        )
        self.update_schema = to_streamed_response_wrapper(
            databases.update_schema,
        )


class AsyncDatabasesResourceWithStreamingResponse:
    def __init__(self, databases: AsyncDatabasesResource) -> None:
        self._databases = databases

        self.update = async_to_streamed_response_wrapper(
            databases.update,
        )
        self.delete_schema = async_to_streamed_response_wrapper(
            databases.delete_schema,
        )
        self.update_schema = async_to_streamed_response_wrapper(
            databases.update_schema,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["WebhookReceiveResponse"]


class WebhookReceiveResponse(BaseModel):
    """Response for successful webhook receipt."""

    delivery_id: str

    event_type: str

    received: bool

    message: Optional[str] = None

    processed: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["GroupCreateParams"]


class GroupCreateParams(TypedDict, total=False):
    name: Required[str]
    """Unique group name within the client"""

    description: Optional[str]
    """Human-readable group description"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from ..client_user import ClientUser

__all__ = ["UserListResponse"]


class UserListResponse(BaseModel):
    """Response schema for listing client users."""

    total: int

    users: List[ClientUser]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["CreateTenantParam"]


class CreateTenantParam(TypedDict, total=False):
    """Request model for creating a single tenant."""

    tenant_key: Required[str]
    """Unique tenant identifier within the client"""

    connection_id: Optional[str]
    """Connection ID (required for DATABASE tenancy type)"""

    database_name: Optional[str]
    """Database name (required for DATABASE tenancy type)"""

    group_names: Optional[SequenceNotStr[str]]
    """List of group names to associate with the tenant"""

    tenant_name: Optional[str]
    """Human-readable tenant name"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["GroupDetail", "Tenant"]


class Tenant(BaseModel):
    """Tenant information in group responses."""

    id: str
    """Tenant ID"""

    name: Optional[str] = None
    """Human-readable tenant name"""

    tenant_key: str
    """Unique tenant identifier"""


class GroupDetail(BaseModel):
    """Response model for a single group (with full tenant list)."""

    id: str
    """Group ID"""

    created_at: datetime
    """Creation timestamp"""

    description: Optional[str] = None
    """Group description"""

    name: str
    """Group name"""

    tenants: List[Tenant]
    """Tenants in this group"""

    updated_at: datetime
    """Last update timestamp"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .batch_tenant_error import BatchTenantError
from .batch_tenant_success import BatchTenantSuccess

__all__ = ["BatchDeleteResponse"]


class BatchDeleteResponse(BaseModel):
    """Response model for batch tenant deletions."""

    failed: List[BatchTenantError]
    """Failed tenant deletions"""

    succeeded: List[BatchTenantSuccess]
    """Successfully deleted tenants"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["GitHubCallbackParams"]


class GitHubCallbackParams(TypedDict, total=False):
    state: Required[str]
    """State token for CSRF validation"""

    code: Optional[str]
    """OAuth authorization code"""

    error: Optional[str]
    """OAuth error code from GitHub"""

    error_description: Optional[str]
    """OAuth error description"""

    installation_id: Optional[int]
    """GitHub App installation ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from ..client_user_role import ClientUserRole

__all__ = ["UserCreateParams"]


class UserCreateParams(TypedDict, total=False):
    email: Required[str]

    password: Required[str]

    first_name: Optional[str]

    last_name: Optional[str]

    role: ClientUserRole
    """Role of a user within a client organization."""

    username: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ConnectionUpdateParams"]


class ConnectionUpdateParams(TypedDict, total=False):
    description: Optional[str]
    """Connection description"""

    label: Optional[str]
    """Human-readable display label"""

    name: Optional[str]
    """Connection name (snake_case identifier)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .readyz import (
    ReadyzResource,
    AsyncReadyzResource,
    ReadyzResourceWithRawResponse,
    AsyncReadyzResourceWithRawResponse,
    ReadyzResourceWithStreamingResponse,
    AsyncReadyzResourceWithStreamingResponse,
)
from .healthz import (
    HealthzResource,
    AsyncHealthzResource,
    HealthzResourceWithRawResponse,
    AsyncHealthzResourceWithRawResponse,
    HealthzResourceWithStreamingResponse,
    AsyncHealthzResourceWithStreamingResponse,
)

__all__ = [
    "V1Resource",
    "AsyncV1Resource",
    "V1ResourceWithRawResponse",
    "AsyncV1ResourceWithRawResponse",
    "V1ResourceWithStreamingResponse",
    "AsyncV1ResourceWithStreamingResponse",
    "HealthzResource",
    "AsyncHealthzResource",
    "HealthzResourceWithRawResponse",
    "AsyncHealthzResourceWithRawResponse",
    "HealthzResourceWithStreamingResponse",
    "AsyncHealthzResourceWithStreamingResponse",
    "ReadyzResource",
    "AsyncReadyzResource",
    "ReadyzResourceWithRawResponse",
    "AsyncReadyzResourceWithRawResponse",
    "ReadyzResourceWithStreamingResponse",
    "AsyncReadyzResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel
from .repository import Repository

__all__ = ["RepoListResponse"]


class RepoListResponse(BaseModel):
    """List of repositories response."""

    repositories: List[Repository]

    total_count: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .repository import Repository

__all__ = ["RepoSelectResponse"]


class RepoSelectResponse(BaseModel):
    """Repository selection response."""

    has_existing_structure: bool

    message: str

    needs_scaffolding: bool

    repository: Repository
    """Repository information response."""

    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .me import (
    MeResource,
    AsyncMeResource,
    MeResourceWithRawResponse,
    AsyncMeResourceWithRawResponse,
    MeResourceWithStreamingResponse,
    AsyncMeResourceWithStreamingResponse,
)
from .v1 import (
    V1Resource,
    AsyncV1Resource,
    V1ResourceWithRawResponse,
    AsyncV1ResourceWithRawResponse,
    V1ResourceWithStreamingResponse,
    AsyncV1ResourceWithStreamingResponse,
)
from .org import (
    OrgResource,
    AsyncOrgResource,
    OrgResourceWithRawResponse,
    AsyncOrgResourceWithRawResponse,
    OrgResourceWithStreamingResponse,
    AsyncOrgResourceWithStreamingResponse,
)
from .github import (
    GitHubResource,
    AsyncGitHubResource,
    GitHubResourceWithRawResponse,
    AsyncGitHubResourceWithRawResponse,
    GitHubResourceWithStreamingResponse,
    AsyncGitHubResourceWithStreamingResponse,
)
from .groups import (
    GroupsResource,
    AsyncGroupsResource,
    GroupsResourceWithRawResponse,
    AsyncGroupsResourceWithRawResponse,
    GroupsResourceWithStreamingResponse,
    AsyncGroupsResourceWithStreamingResponse,
)
from .tenants import (
    TenantsResource,
    AsyncTenantsResource,
    TenantsResourceWithRawResponse,
    AsyncTenantsResourceWithRawResponse,
    TenantsResourceWithStreamingResponse,
    AsyncTenantsResourceWithStreamingResponse,
)
from .connections import (
    ConnectionsResource,
    AsyncConnectionsResource,
    ConnectionsResourceWithRawResponse,
    AsyncConnectionsResourceWithRawResponse,
    ConnectionsResourceWithStreamingResponse,
    AsyncConnectionsResourceWithStreamingResponse,
)

__all__ = [
    "ConnectionsResource",
    "AsyncConnectionsResource",
    "ConnectionsResourceWithRawResponse",
    "AsyncConnectionsResourceWithRawResponse",
    "ConnectionsResourceWithStreamingResponse",
    "AsyncConnectionsResourceWithStreamingResponse",
    "GitHubResource",
    "AsyncGitHubResource",
    "GitHubResourceWithRawResponse",
    "AsyncGitHubResourceWithRawResponse",
    "GitHubResourceWithStreamingResponse",
    "AsyncGitHubResourceWithStreamingResponse",
    "GroupsResource",
    "AsyncGroupsResource",
    "GroupsResourceWithRawResponse",
    "AsyncGroupsResourceWithRawResponse",
    "GroupsResourceWithStreamingResponse",
    "AsyncGroupsResourceWithStreamingResponse",
    "MeResource",
    "AsyncMeResource",
    "MeResourceWithRawResponse",
    "AsyncMeResourceWithRawResponse",
    "MeResourceWithStreamingResponse",
    "AsyncMeResourceWithStreamingResponse",
    "OrgResource",
    "AsyncOrgResource",
    "OrgResourceWithRawResponse",
    "AsyncOrgResourceWithRawResponse",
    "OrgResourceWithStreamingResponse",
    "AsyncOrgResourceWithStreamingResponse",
    "TenantsResource",
    "AsyncTenantsResource",
    "TenantsResourceWithRawResponse",
    "AsyncTenantsResourceWithRawResponse",
    "TenantsResourceWithStreamingResponse",
    "AsyncTenantsResourceWithStreamingResponse",
    "V1Resource",
    "AsyncV1Resource",
    "V1ResourceWithRawResponse",
    "AsyncV1ResourceWithRawResponse",
    "V1ResourceWithStreamingResponse",
    "AsyncV1ResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Iterable, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import required_args, maybe_transform, async_maybe_transform
from .databases import (
    DatabasesResource,
    AsyncDatabasesResource,
    DatabasesResourceWithRawResponse,
    AsyncDatabasesResourceWithRawResponse,
    DatabasesResourceWithStreamingResponse,
    AsyncDatabasesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ....types.v1 import connection_create_params, connection_update_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.connection import Connection
from ....types.v1.database_config_param import DatabaseConfigParam
from ....types.v1.connection_list_response import ConnectionListResponse
from ....types.v1.connection_sync_response import ConnectionSyncResponse
from ....types.v1.connection_retrieve_credential_response import ConnectionRetrieveCredentialResponse

__all__ = ["ConnectionsResource", "AsyncConnectionsResource"]


class ConnectionsResource(SyncAPIResource):
    @cached_property
    def databases(self) -> DatabasesResource:
        return DatabasesResource(self._client)

    @cached_property
    def with_raw_response(self) -> ConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return ConnectionsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["postgresql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: Database host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: Database password

          username: Database username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: Database port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account: str,
        auth: connection_create_params.SnowflakeConnectionConfigAuth,
        databases: Iterable[DatabaseConfigParam],
        name: str,
        role: str,
        username: str,
        warehouse: str,
        warehouse_type: Literal["snowflake"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          account: Snowflake account identifier (e.g., 'xy12345.us-east-1')

          auth: Authentication configuration

          databases: Databases to include in the connection (at least one required)

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          role: Snowflake role

          username: Snowflake username

          warehouse: Compute warehouse name

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        access_token: str,
        databases: Iterable[DatabaseConfigParam],
        http_path: str,
        name: str,
        server_hostname: str,
        warehouse_type: Literal["databricks"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          access_token: Databricks personal access token

          databases: Databases to include in the connection (at least one required)

          http_path: SQL warehouse HTTP path (e.g., '/sql/1.0/warehouses/xxx')

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          server_hostname: Databricks server hostname (e.g., 'dbc-xxx.cloud.databricks.com')

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["clickhouse"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: ClickHouse host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: ClickHouse password

          username: ClickHouse username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: ClickHouse port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["mssql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: SQL Server host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: SQL Server password

          username: SQL Server username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: SQL Server port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["databases", "host", "name", "password", "username", "warehouse_type"],
        ["account", "auth", "databases", "name", "role", "username", "warehouse", "warehouse_type"],
        ["access_token", "databases", "http_path", "name", "server_hostname", "warehouse_type"],
    )
    def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str | Omit = omit,
        name: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        warehouse_type: Literal["postgresql"]
        | Literal["snowflake"]
        | Literal["databricks"]
        | Literal["clickhouse"]
        | Literal["mssql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        account: str | Omit = omit,
        auth: connection_create_params.SnowflakeConnectionConfigAuth | Omit = omit,
        role: str | Omit = omit,
        warehouse: str | Omit = omit,
        access_token: str | Omit = omit,
        http_path: str | Omit = omit,
        server_hostname: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        return self._post(
            "/api/v1/connections",
            body=maybe_transform(
                {
                    "databases": databases,
                    "host": host,
                    "name": name,
                    "password": password,
                    "username": username,
                    "warehouse_type": warehouse_type,
                    "database_timezone": database_timezone,
                    "description": description,
                    "label": label,
                    "port": port,
                    "query_timeout": query_timeout,
                    "query_timezone_conversion": query_timezone_conversion,
                    "account": account,
                    "auth": auth,
                    "role": role,
                    "warehouse": warehouse,
                    "access_token": access_token,
                    "http_path": http_path,
                    "server_hostname": server_hostname,
                },
                connection_create_params.ConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Get a single warehouse connection by kater_id.

        Returns connection from the database (source of truth) with full hierarchy.
        Supports content negotiation via Accept header (handled by MultiFormatRoute):

        - application/json (default): Returns JSON response
        - application/yaml: Returns YAML representation

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: NotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._get(
            f"/api/v1/connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    def update(
        self,
        connection_id: str,
        *,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Update a warehouse connection's metadata.

        Updates name, label, and/or description fields.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If connection doesn't exist (404)

        Args:
          description: Connection description

          label: Human-readable display label

          name: Connection name (snake_case identifier)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._patch(
            f"/api/v1/connections/{connection_id}",
            body=maybe_transform(
                {
                    "description": description,
                    "label": label,
                    "name": name,
                },
                connection_update_params.ConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionListResponse:
        """
        List all warehouse connections for the client.

        Returns connections from the database joined with schema information from
        GitHub. Connections with pending PRs will have null GitHub fields until merged.
        Returns empty list if GitHub is not configured.

        RLS: Filtered to current client (DualClientRLSDB).
        """
        return self._get(
            "/api/v1/connections",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionListResponse,
        )

    def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a warehouse connection.

        Soft-deletes the connection record and cascades to all associated databases and
        schemas, setting deleted_at and deleted_by fields.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/v1/connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve_credential(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveCredentialResponse:
        """
        Get decrypted warehouse credentials for a connection.

        Returns the decrypted credentials for a connection. This is sensitive data and
        should only be used when actually needed for warehouse operations.

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: NotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return cast(
            ConnectionRetrieveCredentialResponse,
            self._get(
                f"/api/v1/connections/{connection_id}/credential",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ConnectionRetrieveCredentialResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def sync(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionSyncResponse:
        """
        Sync view schemas from warehouse and create a PR (or update existing).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return self._post(
            f"/api/v1/connections/{connection_id}/sync",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionSyncResponse,
        )


class AsyncConnectionsResource(AsyncAPIResource):
    @cached_property
    def databases(self) -> AsyncDatabasesResource:
        return AsyncDatabasesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncConnectionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncConnectionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConnectionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncConnectionsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["postgresql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: Database host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: Database password

          username: Database username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: Database port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account: str,
        auth: connection_create_params.SnowflakeConnectionConfigAuth,
        databases: Iterable[DatabaseConfigParam],
        name: str,
        role: str,
        username: str,
        warehouse: str,
        warehouse_type: Literal["snowflake"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          account: Snowflake account identifier (e.g., 'xy12345.us-east-1')

          auth: Authentication configuration

          databases: Databases to include in the connection (at least one required)

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          role: Snowflake role

          username: Snowflake username

          warehouse: Compute warehouse name

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        access_token: str,
        databases: Iterable[DatabaseConfigParam],
        http_path: str,
        name: str,
        server_hostname: str,
        warehouse_type: Literal["databricks"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          access_token: Databricks personal access token

          databases: Databases to include in the connection (at least one required)

          http_path: SQL warehouse HTTP path (e.g., '/sql/1.0/warehouses/xxx')

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          server_hostname: Databricks server hostname (e.g., 'dbc-xxx.cloud.databricks.com')

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["clickhouse"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: ClickHouse host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: ClickHouse password

          username: ClickHouse username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: ClickHouse port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str,
        name: str,
        password: str,
        username: str,
        warehouse_type: Literal["mssql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Create a new warehouse connection.

        Args:
          databases: Databases to include in the connection (at least one required)

          host: SQL Server host

          name: Name of the connection (snake_case: lowercase letters, numbers, underscores)

          password: SQL Server password

          username: SQL Server username

          warehouse_type: Warehouse type

          database_timezone: Default timezone for the connection (e.g., 'UTC', 'America/New_York')

          description: Description of the connection

          label: Human-readable label for the connection (defaults to name if not set)

          port: SQL Server port

          query_timeout: Query timeout in seconds (1-3600)

          query_timezone_conversion: Timezone conversion mode: 'do_not_convert' or 'convert_to_utc'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(
        ["databases", "host", "name", "password", "username", "warehouse_type"],
        ["account", "auth", "databases", "name", "role", "username", "warehouse", "warehouse_type"],
        ["access_token", "databases", "http_path", "name", "server_hostname", "warehouse_type"],
    )
    async def create(
        self,
        *,
        databases: Iterable[DatabaseConfigParam],
        host: str | Omit = omit,
        name: str,
        password: str | Omit = omit,
        username: str | Omit = omit,
        warehouse_type: Literal["postgresql"]
        | Literal["snowflake"]
        | Literal["databricks"]
        | Literal["clickhouse"]
        | Literal["mssql"],
        database_timezone: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        port: int | Omit = omit,
        query_timeout: Optional[int] | Omit = omit,
        query_timezone_conversion: Optional[str] | Omit = omit,
        account: str | Omit = omit,
        auth: connection_create_params.SnowflakeConnectionConfigAuth | Omit = omit,
        role: str | Omit = omit,
        warehouse: str | Omit = omit,
        access_token: str | Omit = omit,
        http_path: str | Omit = omit,
        server_hostname: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        return await self._post(
            "/api/v1/connections",
            body=await async_maybe_transform(
                {
                    "databases": databases,
                    "host": host,
                    "name": name,
                    "password": password,
                    "username": username,
                    "warehouse_type": warehouse_type,
                    "database_timezone": database_timezone,
                    "description": description,
                    "label": label,
                    "port": port,
                    "query_timeout": query_timeout,
                    "query_timezone_conversion": query_timezone_conversion,
                    "account": account,
                    "auth": auth,
                    "role": role,
                    "warehouse": warehouse,
                    "access_token": access_token,
                    "http_path": http_path,
                    "server_hostname": server_hostname,
                },
                connection_create_params.ConnectionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    async def retrieve(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Get a single warehouse connection by kater_id.

        Returns connection from the database (source of truth) with full hierarchy.
        Supports content negotiation via Accept header (handled by MultiFormatRoute):

        - application/json (default): Returns JSON response
        - application/yaml: Returns YAML representation

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: NotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._get(
            f"/api/v1/connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    async def update(
        self,
        connection_id: str,
        *,
        description: Optional[str] | Omit = omit,
        label: Optional[str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Connection:
        """
        Update a warehouse connection's metadata.

        Updates name, label, and/or description fields.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If connection doesn't exist (404)

        Args:
          description: Connection description

          label: Human-readable display label

          name: Connection name (snake_case identifier)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._patch(
            f"/api/v1/connections/{connection_id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "label": label,
                    "name": name,
                },
                connection_update_params.ConnectionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Connection,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionListResponse:
        """
        List all warehouse connections for the client.

        Returns connections from the database joined with schema information from
        GitHub. Connections with pending PRs will have null GitHub fields until merged.
        Returns empty list if GitHub is not configured.

        RLS: Filtered to current client (DualClientRLSDB).
        """
        return await self._get(
            "/api/v1/connections",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionListResponse,
        )

    async def delete(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a warehouse connection.

        Soft-deletes the connection record and cascades to all associated databases and
        schemas, setting deleted_at and deleted_by fields.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: ConnectionNotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/v1/connections/{connection_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve_credential(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionRetrieveCredentialResponse:
        """
        Get decrypted warehouse credentials for a connection.

        Returns the decrypted credentials for a connection. This is sensitive data and
        should only be used when actually needed for warehouse operations.

        RLS: Filtered to current client (DualClientRLSDB).

        Raises: NotFoundError: If connection doesn't exist (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return cast(
            ConnectionRetrieveCredentialResponse,
            await self._get(
                f"/api/v1/connections/{connection_id}/credential",
                options=make_request_options(
                    extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
                ),
                cast_to=cast(
                    Any, ConnectionRetrieveCredentialResponse
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    async def sync(
        self,
        connection_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ConnectionSyncResponse:
        """
        Sync view schemas from warehouse and create a PR (or update existing).

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connection_id:
            raise ValueError(f"Expected a non-empty value for `connection_id` but received {connection_id!r}")
        return await self._post(
            f"/api/v1/connections/{connection_id}/sync",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConnectionSyncResponse,
        )


class ConnectionsResourceWithRawResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.create = to_raw_response_wrapper(
            connections.create,
        )
        self.retrieve = to_raw_response_wrapper(
            connections.retrieve,
        )
        self.update = to_raw_response_wrapper(
            connections.update,
        )
        self.list = to_raw_response_wrapper(
            connections.list,
        )
        self.delete = to_raw_response_wrapper(
            connections.delete,
        )
        self.retrieve_credential = to_raw_response_wrapper(
            connections.retrieve_credential,
        )
        self.sync = to_raw_response_wrapper(
            connections.sync,
        )

    @cached_property
    def databases(self) -> DatabasesResourceWithRawResponse:
        return DatabasesResourceWithRawResponse(self._connections.databases)


class AsyncConnectionsResourceWithRawResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.create = async_to_raw_response_wrapper(
            connections.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            connections.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            connections.update,
        )
        self.list = async_to_raw_response_wrapper(
            connections.list,
        )
        self.delete = async_to_raw_response_wrapper(
            connections.delete,
        )
        self.retrieve_credential = async_to_raw_response_wrapper(
            connections.retrieve_credential,
        )
        self.sync = async_to_raw_response_wrapper(
            connections.sync,
        )

    @cached_property
    def databases(self) -> AsyncDatabasesResourceWithRawResponse:
        return AsyncDatabasesResourceWithRawResponse(self._connections.databases)


class ConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: ConnectionsResource) -> None:
        self._connections = connections

        self.create = to_streamed_response_wrapper(
            connections.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            connections.update,
        )
        self.list = to_streamed_response_wrapper(
            connections.list,
        )
        self.delete = to_streamed_response_wrapper(
            connections.delete,
        )
        self.retrieve_credential = to_streamed_response_wrapper(
            connections.retrieve_credential,
        )
        self.sync = to_streamed_response_wrapper(
            connections.sync,
        )

    @cached_property
    def databases(self) -> DatabasesResourceWithStreamingResponse:
        return DatabasesResourceWithStreamingResponse(self._connections.databases)


class AsyncConnectionsResourceWithStreamingResponse:
    def __init__(self, connections: AsyncConnectionsResource) -> None:
        self._connections = connections

        self.create = async_to_streamed_response_wrapper(
            connections.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            connections.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            connections.update,
        )
        self.list = async_to_streamed_response_wrapper(
            connections.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            connections.delete,
        )
        self.retrieve_credential = async_to_streamed_response_wrapper(
            connections.retrieve_credential,
        )
        self.sync = async_to_streamed_response_wrapper(
            connections.sync,
        )

    @cached_property
    def databases(self) -> AsyncDatabasesResourceWithStreamingResponse:
        return AsyncDatabasesResourceWithStreamingResponse(self._connections.databases)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.v1.client_user import ClientUser

__all__ = ["MeResource", "AsyncMeResource"]


class MeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return MeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return MeResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClientUser:
        """
        Get the current authenticated user's information.

        Returns the user's profile within their active organization. This endpoint only
        requires authentication - no special permissions needed.

        RLS: Filtered to current user's data only (RLSDB).
        """
        return self._get(
            "/api/v1/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClientUser,
        )


class AsyncMeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncMeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncMeResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClientUser:
        """
        Get the current authenticated user's information.

        Returns the user's profile within their active organization. This endpoint only
        requires authentication - no special permissions needed.

        RLS: Filtered to current user's data only (RLSDB).
        """
        return await self._get(
            "/api/v1/me",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClientUser,
        )


class MeResourceWithRawResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_raw_response_wrapper(
            me.retrieve,
        )


class AsyncMeResourceWithRawResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_raw_response_wrapper(
            me.retrieve,
        )


class MeResourceWithStreamingResponse:
    def __init__(self, me: MeResource) -> None:
        self._me = me

        self.retrieve = to_streamed_response_wrapper(
            me.retrieve,
        )


class AsyncMeResourceWithStreamingResponse:
    def __init__(self, me: AsyncMeResource) -> None:
        self._me = me

        self.retrieve = async_to_streamed_response_wrapper(
            me.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["TenancyType"]

TenancyType: TypeAlias = Literal["row", "database", "none"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["BatchTenantError"]


class BatchTenantError(BaseModel):
    """Failed tenant operation in batch response."""

    code: str
    """Error code"""

    message: str
    """Error message"""

    index: Optional[int] = None
    """Index in request array (for create operations)"""

    tenant_id: Optional[str] = None
    """Tenant ID (for update/delete operations)"""

    tenant_key: Optional[str] = None
    """Tenant key (for create operations)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .repos import (
    ReposResource,
    AsyncReposResource,
    ReposResourceWithRawResponse,
    AsyncReposResourceWithRawResponse,
    ReposResourceWithStreamingResponse,
    AsyncReposResourceWithStreamingResponse,
)
from .github import (
    GitHubResource,
    AsyncGitHubResource,
    GitHubResourceWithRawResponse,
    AsyncGitHubResourceWithRawResponse,
    GitHubResourceWithStreamingResponse,
    AsyncGitHubResourceWithStreamingResponse,
)
from .scaffold import (
    ScaffoldResource,
    AsyncScaffoldResource,
    ScaffoldResourceWithRawResponse,
    AsyncScaffoldResourceWithRawResponse,
    ScaffoldResourceWithStreamingResponse,
    AsyncScaffoldResourceWithStreamingResponse,
)
from .webhooks import (
    WebhooksResource,
    AsyncWebhooksResource,
    WebhooksResourceWithRawResponse,
    AsyncWebhooksResourceWithRawResponse,
    WebhooksResourceWithStreamingResponse,
    AsyncWebhooksResourceWithStreamingResponse,
)

__all__ = [
    "ReposResource",
    "AsyncReposResource",
    "ReposResourceWithRawResponse",
    "AsyncReposResourceWithRawResponse",
    "ReposResourceWithStreamingResponse",
    "AsyncReposResourceWithStreamingResponse",
    "ScaffoldResource",
    "AsyncScaffoldResource",
    "ScaffoldResourceWithRawResponse",
    "AsyncScaffoldResourceWithRawResponse",
    "ScaffoldResourceWithStreamingResponse",
    "AsyncScaffoldResourceWithStreamingResponse",
    "WebhooksResource",
    "AsyncWebhooksResource",
    "WebhooksResourceWithRawResponse",
    "AsyncWebhooksResourceWithRawResponse",
    "WebhooksResourceWithStreamingResponse",
    "AsyncWebhooksResourceWithStreamingResponse",
    "GitHubResource",
    "AsyncGitHubResource",
    "GitHubResourceWithRawResponse",
    "AsyncGitHubResourceWithRawResponse",
    "GitHubResourceWithStreamingResponse",
    "AsyncGitHubResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.org import TenancyType, client_update_params
from ....types.v1.org.client import Client
from ....types.v1.org.tenancy_type import TenancyType

__all__ = ["ClientResource", "AsyncClientResource"]


class ClientResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClientResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ClientResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClientResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return ClientResourceWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Client:
        """
        Get the current user's organization details.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return self._get(
            "/api/v1/org/client",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    def update(
        self,
        *,
        can_auto_join_by_domain: Optional[bool] | Omit = omit,
        domain: Optional[str] | Omit = omit,
        logo_url: Optional[str] | Omit = omit,
        members_must_have_matching_domain: Optional[bool] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tenancy_type: Optional[TenancyType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Client:
        """
        Update the current user's organization details.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenancy_type: Type of tenancy for a client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/api/v1/org/client",
            body=maybe_transform(
                {
                    "can_auto_join_by_domain": can_auto_join_by_domain,
                    "domain": domain,
                    "logo_url": logo_url,
                    "members_must_have_matching_domain": members_must_have_matching_domain,
                    "name": name,
                    "tenancy_type": tenancy_type,
                },
                client_update_params.ClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )


class AsyncClientResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClientResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncClientResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClientResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncClientResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Client:
        """
        Get the current user's organization details.

        RLS: Filtered to current client (ClientRLSDB).
        """
        return await self._get(
            "/api/v1/org/client",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )

    async def update(
        self,
        *,
        can_auto_join_by_domain: Optional[bool] | Omit = omit,
        domain: Optional[str] | Omit = omit,
        logo_url: Optional[str] | Omit = omit,
        members_must_have_matching_domain: Optional[bool] | Omit = omit,
        name: Optional[str] | Omit = omit,
        tenancy_type: Optional[TenancyType] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Client:
        """
        Update the current user's organization details.

        RLS: Filtered to current client (ClientRLSDB).

        Args:
          tenancy_type: Type of tenancy for a client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/api/v1/org/client",
            body=await async_maybe_transform(
                {
                    "can_auto_join_by_domain": can_auto_join_by_domain,
                    "domain": domain,
                    "logo_url": logo_url,
                    "members_must_have_matching_domain": members_must_have_matching_domain,
                    "name": name,
                    "tenancy_type": tenancy_type,
                },
                client_update_params.ClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Client,
        )


class ClientResourceWithRawResponse:
    def __init__(self, client: ClientResource) -> None:
        self._client = client

        self.retrieve = to_raw_response_wrapper(
            client.retrieve,
        )
        self.update = to_raw_response_wrapper(
            client.update,
        )


class AsyncClientResourceWithRawResponse:
    def __init__(self, client: AsyncClientResource) -> None:
        self._client = client

        self.retrieve = async_to_raw_response_wrapper(
            client.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            client.update,
        )


class ClientResourceWithStreamingResponse:
    def __init__(self, client: ClientResource) -> None:
        self._client = client

        self.retrieve = to_streamed_response_wrapper(
            client.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            client.update,
        )


class AsyncClientResourceWithStreamingResponse:
    def __init__(self, client: AsyncClientResource) -> None:
        self._client = client

        self.retrieve = async_to_streamed_response_wrapper(
            client.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            client.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from kater import Kater, AsyncKater
from tests.utils import assert_matches_type
from kater.types.v1.connections import (
    DatabaseUpdateResponse,
    DatabaseUpdateSchemaResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestDatabases:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Kater) -> None:
        database = client.v1.connections.databases.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Kater) -> None:
        database = client.v1.connections.databases.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            auto_merge=True,
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Kater) -> None:
        response = client.v1.connections.databases.with_raw_response.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = response.parse()
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Kater) -> None:
        with client.v1.connections.databases.with_streaming_response.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = response.parse()
            assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.databases.with_raw_response.update(
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            client.v1.connections.databases.with_raw_response.update(
                database_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete_schema(self, client: Kater) -> None:
        database = client.v1.connections.databases.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert database is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete_schema(self, client: Kater) -> None:
        response = client.v1.connections.databases.with_raw_response.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = response.parse()
        assert database is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete_schema(self, client: Kater) -> None:
        with client.v1.connections.databases.with_streaming_response.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = response.parse()
            assert database is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete_schema(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `schema_id` but received ''"):
            client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_schema(self, client: Kater) -> None:
        database = client.v1.connections.databases.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_schema_with_all_params(self, client: Kater) -> None:
        database = client.v1.connections.databases.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            auto_merge=True,
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update_schema(self, client: Kater) -> None:
        response = client.v1.connections.databases.with_raw_response.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = response.parse()
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update_schema(self, client: Kater) -> None:
        with client.v1.connections.databases.with_streaming_response.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = response.parse()
            assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update_schema(self, client: Kater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `schema_id` but received ''"):
            client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )


class TestAsyncDatabases:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncKater) -> None:
        database = await async_client.v1.connections.databases.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncKater) -> None:
        database = await async_client.v1.connections.databases.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            auto_merge=True,
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.databases.with_raw_response.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = await response.parse()
        assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.databases.with_streaming_response.update(
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = await response.parse()
            assert_matches_type(DatabaseUpdateResponse, database, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.update(
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.update(
                database_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete_schema(self, async_client: AsyncKater) -> None:
        database = await async_client.v1.connections.databases.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert database is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete_schema(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.databases.with_raw_response.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = await response.parse()
        assert database is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete_schema(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.databases.with_streaming_response.delete_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = await response.parse()
            assert database is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete_schema(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `schema_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.delete_schema(
                schema_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_schema(self, async_client: AsyncKater) -> None:
        database = await async_client.v1.connections.databases.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_schema_with_all_params(self, async_client: AsyncKater) -> None:
        database = await async_client.v1.connections.databases.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            auto_merge=True,
            description="description",
            label="label",
            name="name",
        )
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update_schema(self, async_client: AsyncKater) -> None:
        response = await async_client.v1.connections.databases.with_raw_response.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        database = await response.parse()
        assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update_schema(self, async_client: AsyncKater) -> None:
        async with async_client.v1.connections.databases.with_streaming_response.update_schema(
            schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            database = await response.parse()
            assert_matches_type(DatabaseUpdateSchemaResponse, database, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update_schema(self, async_client: AsyncKater) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `connection_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `database_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `schema_id` but received ''"):
            await async_client.v1.connections.databases.with_raw_response.update_schema(
                schema_id="",
                connection_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
                database_id="182bd5e5-6e1a-4fe4-a799-aa6d9a6ab26e",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .readyz_check_response import ReadyzCheckResponse as ReadyzCheckResponse
from .healthz_check_response import HealthzCheckResponse as HealthzCheckResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, SequenceNotStr, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.v1.groups import tenant_add_params
from ....types.v1.groups.tenant_add_response import TenantAddResponse

__all__ = ["TenantsResource", "AsyncTenantsResource"]


class TenantsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return TenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return TenantsResourceWithStreamingResponse(self)

    def add(
        self,
        group_id: str,
        *,
        tenant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantAddResponse:
        """Add tenants to a group.

        Accepts a list of tenant IDs.

        Returns detailed results showing which tenants
        were added, which were already members, and which don't exist.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: GroupNotFoundError: If group doesn't exist (404)

        Args:
          tenant_ids: List of tenant IDs to add to the group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return self._post(
            f"/api/v1/groups/{group_id}/tenants",
            body=maybe_transform({"tenant_ids": tenant_ids}, tenant_add_params.TenantAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantAddResponse,
        )

    def remove(
        self,
        tenant_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a tenant from a group.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: GroupNotFoundError: If group doesn't exist (404) TenantNotFoundError: If
        tenant is not a member of the group (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/api/v1/groups/{group_id}/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncTenantsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTenantsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncTenantsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTenantsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/kater-ai/kater-python-sdk#with_streaming_response
        """
        return AsyncTenantsResourceWithStreamingResponse(self)

    async def add(
        self,
        group_id: str,
        *,
        tenant_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TenantAddResponse:
        """Add tenants to a group.

        Accepts a list of tenant IDs.

        Returns detailed results showing which tenants
        were added, which were already members, and which don't exist.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: GroupNotFoundError: If group doesn't exist (404)

        Args:
          tenant_ids: List of tenant IDs to add to the group

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        return await self._post(
            f"/api/v1/groups/{group_id}/tenants",
            body=await async_maybe_transform({"tenant_ids": tenant_ids}, tenant_add_params.TenantAddParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TenantAddResponse,
        )

    async def remove(
        self,
        tenant_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove a tenant from a group.

        RLS: Filtered to current client (ClientRLSDB).

        Raises: GroupNotFoundError: If group doesn't exist (404) TenantNotFoundError: If
        tenant is not a member of the group (404)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not tenant_id:
            raise ValueError(f"Expected a non-empty value for `tenant_id` but received {tenant_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/api/v1/groups/{group_id}/tenants/{tenant_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class TenantsResourceWithRawResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.add = to_raw_response_wrapper(
            tenants.add,
        )
        self.remove = to_raw_response_wrapper(
            tenants.remove,
        )


class AsyncTenantsResourceWithRawResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.add = async_to_raw_response_wrapper(
            tenants.add,
        )
        self.remove = async_to_raw_response_wrapper(
            tenants.remove,
        )


class TenantsResourceWithStreamingResponse:
    def __init__(self, tenants: TenantsResource) -> None:
        self._tenants = tenants

        self.add = to_streamed_response_wrapper(
            tenants.add,
        )
        self.remove = to_streamed_response_wrapper(
            tenants.remove,
        )


class AsyncTenantsResourceWithStreamingResponse:
    def __init__(self, tenants: AsyncTenantsResource) -> None:
        self._tenants = tenants

        self.add = async_to_streamed_response_wrapper(
            tenants.add,
        )
        self.remove = async_to_streamed_response_wrapper(
            tenants.remove,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..create_tenant_param import CreateTenantParam

__all__ = ["BatchCreateParams"]


class BatchCreateParams(TypedDict, total=False):
    tenants: Required[Iterable[CreateTenantParam]]
    """List of tenants to create"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["GitHubConnectResponse"]


class GitHubConnectResponse(BaseModel):
    """Response for starting OAuth flow."""

    authorization_url: str

    state: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .repository import Repository as Repository
from .scaffold_trigger import ScaffoldTrigger as ScaffoldTrigger
from .repo_list_response import RepoListResponse as RepoListResponse
from .repo_select_params import RepoSelectParams as RepoSelectParams
from .repo_select_response import RepoSelectResponse as RepoSelectResponse
from .webhook_receive_response import WebhookReceiveResponse as WebhookReceiveResponse
